% This procedure generates results in Table 4 of the paper: Inference for VARs Identified Restrictions, draft January 2013 

%---    INITIALIZATION
T = 100;
N_sim = 1000; %# samples of artificial data; used also to estimate lambda
nirf = 5; %#maximum horizon for irf
nirf_rest = 5; % horizons for which are sign restrictions are imposed; horizon 1 is the contemporaneous responses 

seed = 5;
ndrawscv = 100000; %# draws for simulated cv
alpha = 0.1;
alpha1 = 0.05;
alpha2 = alpha - alpha1;


%design of GDP
const = [0  0];
phi = [0.873 0.003; -0.229 0.230];  %design 2 NBER wp Table 1.
sigma = [0.087  -0.027; -0.027 0.640];  %design 2 NBER wp Table 1.

% phi = [0.806 0.032; -0.278 0.985];  %design 3 NBER wp Table 1.
% sigma = [0.080  -0.023; -0.023 0.674];  %design 3 NBER wp Table 1.

% phi = [0.450 0.014; 0.060 0.953];  %design 4 NBER wp Table 1.
% sigma = [0.044  -0.0093; -0.009 0.296];  %design 4 NBER wp Table 1.

%Sign restrictions
nvar = size(phi,1);

sigmaL = chol(sigma,'lower');
YYirf = estimate_irfs(phi,sigmaL, nirf);
R1 = YYirf(1:nirf_rest,:); % Reduced form impulse response that are sign restricted or part of theta
phivec = reshape(R1',size(YYirf,1)*size(YYirf,2),1);
phivec = [phivec(1,1);phivec(3:end,1)]; %take out the zero element due to the impact IRF
m = size(phivec,1);

IS = setestimatorv2(phivec); %identified set

kappaT = 1.96*log(log(T));
    

stepsize = 0.01;
angle = [-pi:stepsize:pi];

stepsize_theta = 0.001;
        
    for nsim = 1:N_sim
       
        %% (1) generate data
        [Y] =  gdp_VAR(const,phi,sigma,T,seed,nsim); %generate data        
       
        Zm = randn(m,500); %used to compute cv
        
        %  (2) estimate VAR(1) no constant and construct vector phi
        yy = Y(2:T,:);
        xx = Y(1:T-1,:);
        phi_hat = yy'*xx*inv(xx'*xx);
        resid = yy - xx*phi_hat';
        l = ones(T-1,1);
        M_l = eye(T-1) - l*inv(l'*l)*l';
        sigma_hat = resid'*M_l*resid/(T-2);
        clear yy xx resid
        sigmaL_hat = chol(sigma_hat,'lower');
        YYirf_hat = estimate_irfs(phi_hat,sigmaL_hat, nirf); %obtain IRF
        R1_hat = YYirf_hat(1:nirf_rest,:);
        phihatvec = reshape(R1_hat', size(YYirf_hat,1)*size(YYirf_hat,2), 1);
        phihatvec = [phihatvec(1,1);phihatvec(3:end,1)];%take out the zero element due to the impact IRF
        

        %   (3) estimate lambda
        lambda_hat = est_lambdav2(const, phi_hat, sigma_hat, T, seed, N_sim, phihatvec,nirf_rest, nirf);
        Lhat = chol(lambda_hat,'lower');
           
        
        q1_vec = cos(angle);
        q2_vec = sin(angle);

        
        % (4) compute Bonferroni CS 
        
        CSBI(:,nsim) = cs_bonferroni(Zm,phihatvec,lambda_hat,q1_vec,q2_vec,nvar,T, kappaT, alpha1, alpha2);


        % (5) compute projection confidence set

        true = 0;%have you found theta for the projection approach with Bhat=identity matrix?
        theta = CSBI(2,nsim)*0.75; %initialize theta 
        
        while true ==0 %start loop over values of theta
            
            for s = 1:size(angle,2)
                
                    q1 = q1_vec(1,s);
                    q2 = q2_vec(1,s);
                %%------------------------------------------------------------
                %MOMENT SELECTION APPROACH%
                %step 1: objective function for moment selection
                Sq = [[q1, zeros(1,m-1)];[zeros((m-1)/nvar,1),kron(eye((m-1)/nvar),[q1 q2])]];
                Shat = Sq*lambda_hat*Sq';
                Dhat = diag(diag(Shat));
                BhatI = eye(size(Sq ,1));   
                
                    Z = Sq*phihatvec;
 
                      mu_hatI =   (Z>=0).*Z;
                      mu_hatI(1,1) = theta;
                      GTI(1,s) = T*((diag(diag(Dhat).^(-1/2))*(Z-mu_hatI))'*BhatI*(diag(diag(Dhat).^(-1/2))*(Z-mu_hatI))); %objective function for moment selection 
                        
                    
                    if theta ~= 0 && (q1*phihatvec(1,1))~=0  
                        GTpI(1,s) = GTI(1,s);
                       
                    elseif theta ~= 0 && (q1*phihatvec(1,1))==0
                        GTpI(1,s) = GTI(1,s)+ T; %add penalty term if theta~= 0 and first inequality is binding 

                    end
               
                %step 2: compute the cv for the moment selection approach  
                if theta == CSBI(2,nsim)*0.75 %compute cv only once for every generated sample of data
                    %compute the cv for the moment selection approach once for every generated sample of data                                               
                     
                          [cv_momI(1,s)] = cv_projection(Dhat,Sq,Lhat,Zm,alpha,BhatI,T,kappaT,phihatvec);

 
                  %%-------------------------------------------------------------   
                    
                end %end of if theta == CSBI(2,nsim)*0.75 
                
                
                
            end  %end of loop over angle
            
            

        
           G_diffI = min(GTpI - cv_momI); %moment projection approach with Bhat = identity matrix
           

           if G_diffI>=0 && true==0
               CSmomI(1,nsim) = theta;
               true = 1;
           end
       

           theta = theta + stepsize_theta;
            
        end %end of loop over values of theta
      


        clear  phihatvec mu_hat Q_theta cv_momI  G_diffI  GTpI CSqO CSqI 
        
    end %end of loop for nsim
   

tempmomI = (CSmomI-(IS(2,1)*ones(1,N_sim)) > 0);
cov_probmomI = (sum(tempmomI))/N_sim;
lenghtmomI = mean(CSmomI);


tempmomBI = (CSBI(2,:)-(IS(2,1)*ones(1,N_sim)) > 0);
cov_probBI = (sum(tempmomBI))/N_sim;
lenghtBI = mean(CSBI(2,:)-CSBI(1,:));


Table = [{'Table ','cov prob', 'lenght'};
    'Mom Iden',num2cell(cov_probmomI), num2cell(lenghtmomI);

    'Bonf Iden', num2cell(cov_probBI), num2cell(lenghtBI);

    ];