% This procedure generates the results in Table 3, first column of the paper Inference for SVARs Identified with Sign Restrictions, draft January 2013. 

%---    INITIALIZATION
T = 100;
N_sim = 1000;

seed = 2.453745165;
seedcv = 3.85668;
ndrawscv = 100000; %# draws for simulated cv
alpha = 0.1;
alpha1 = 0.05; %for Bonferroni approach
alpha2 = alpha-alpha1; %for Bonferroni approach

    %CRITICAL VALUES:
    randn('state',seedcv);
    z = randn(1,ndrawscv);
    chi2 = chi2rnd(1,1,ndrawscv); 
   
    indexp = (z>=0); %indicator function: takes value one when z is positive
    indexn = (z<0); %indicator function: takes value one when z is negative

    cvB1 = prctile(indexn.*(z.^2),(1-alpha1)*100);
    cvB2 = norminv(1-alpha2/2,0,1);
    
    cvmom1 =  chi2inv(1-alpha,1);
    cvmom2 =  prctile(indexp.*(z.^2),(1-alpha)*100);
    cv_fix = prctile(chi2 + indexp.*(z.^2),(1-alpha)*100); %fixed cv  %if z>=0, add the square of the draw from the standard normal to the draw from the chi2
   
    kappaT = 1.96*log(log(T));
    CS_phi = 0; %initialize counter to compute coverage probability of the CI for the reduced form parameter  

  
%population parameter values
const = zeros(1,2);
phi = zeros(2,2);
sigma = [0.356  -0.122; -0.122 0.701];  %design 1 NBER wp Table 1.
sigmaL = chol(sigma,'lower');
phivec = reshape(sigmaL',size(sigma,1)*size(sigma,2),1);
phivec = [phivec(1,1);phivec(3:4,1)]; %take out the zero element;

 if (phivec(2,1) >= 0)
        IS = phivec(1,1)*max(1,sqrt((phivec(3,1)^2)/(phivec(2,1)^2 + phivec(3,1)^2)));
        
  else
        IS = phivec(1,1)*sqrt((phivec(3,1)^2)/(phivec(2,1)^2 +phivec(3,1)^2));
        
 end
 
stepsize = 0.01; %stepsize of grid for angle, preferred stepsize = 0.01;
stepsize_theta = 0.001; %stepsize of grid for theta 

[Fq_l,Fq_u] = setestimator_q(phivec, stepsize);
IS_q =[acos(Fq_u), acos(Fq_l)]; %needed to compute lenght of the identified set for q


for nsim = 1:N_sim %loop over different generated samples
    
    %% generate data 
    
    [Y] =  gdp_VAR(const,phi,sigma,T,seed,nsim); %generate data (set phi =0 for a VAR(0))
    
    Z3 = randn(3,500);
    
    disp(nsim)
    %estimate VAR(0) no constant
        l = ones(T,1);
        M_l = eye(T) - l*inv(l'*l)*l';
        sigma_hat = Y'*M_l*Y/(T-1);
        sigmaL_hat = chol(sigma_hat,'lower');
        phihatvec = reshape(sigmaL_hat, size(sigma_hat,1)*size(sigma_hat,2), 1);
        phihatvec = [phihatvec(1:2,1);phihatvec(4,1)];
        phihat1 =  phihatvec(1,1);
        phihat2 =  phihatvec(2,1);
        phihat3 =  phihatvec(3,1);
        
        
        %estimate lambda
        lambdahat = est_lambda(const, phi, sigma_hat, T, seed, N_sim, phihatvec);
       
        if T*(phihatvec-phivec)'*inv(lambdahat)*(phihatvec-phivec)<= chi2inv(1-alpha,size(phivec,1)) %compute the coverage probabilities of the CI of the reduced form parameter
            CS_phi = CS_phi+1;
        end
        
    %----  BONFERRONI APPROACH   --------------
    
   %part one of Bonferroni: construct cs_gis of phi
   
   [CSB_O(:,nsim),CSB_I(:,nsim),CSq_O(:,nsim), CSq_I(:,nsim),arc_O(:,nsim),arc_I(:,nsim)] = cs_gis(stepsize, lambdahat, phihatvec, T,  kappaT, cvB1, cvB2, Z3, alpha1);

%     %----- PROJECTION APPROACH  ------------------------------------
%         %% Estimated Identified Set 
    if (phihat2 >= 0)
        theta_max_hat = phihat1*max(1,sqrt((phihat3^2)/(phihat2^2 + phihat3^2)));
        
    else
        theta_max_hat = phihat1*sqrt((phihat3^2)/(phihat2^2 + phihat3^2));
        
    end
    
    IShat(1,nsim) = theta_max_hat;
    

    %Moment Inequality; the cv do not depend on theta
    [cvmom_O,cvmom_I] = cv_mom(T, phihatvec, lambdahat, kappaT, alpha, cv_fix, cvmom1, cvmom2, stepsize, Z3); %compute CS(theta) for binding conditions (G_diff1) and simulation (G_diff2)
    
    true = 0;%have you found confidence set for both approaches (main and conservative cv)?
    true1 = 0;%have you found theta for the conservative cv approach?
    true2 = 0;%have you found theta for the main moment inequality selection approach?
    theta = theta_max_hat;
    
    while true ==0 %loop over theta, starting at the upper bound of the estimated identified set
         %objective function
          [G_thetaO,G_thetaI]  = f_G(stepsize, T, lambdahat, phihatvec, theta ); %G_theta is a vector, collects the value of the function G_theta for every value of angle
        
   
           G_diffI = min(G_thetaI - cvmom_I); %Bhat is identity matrix; find min of the difference over all values of q
        
           G_diffO = min(G_thetaO - cvmom_O);%Bhat is inverse of Omega;find min of the difference over all values of q
           
           if G_diffI>=0 && true1==0  %store largest value of theta that satisfies equation (34) in January 2013 draft
               CSmomI(1,nsim) =theta;
               true1 = 1;
           end
           if G_diffO>=0 && true2==0
               CSmomO(1,nsim) =theta;
               true2 = 1;
           end
       
       if G_diffI>=0 && G_diffO>=0
           true = 1;
       end
      theta = theta + stepsize_theta; 
   end %end loop for theta
   
   clear G_thetaO G_thetaI G_diffI G_diffO

end %end of loop for nsim

CS_phi= CS_phi/N_sim; %coverage probability for the reduced form parameters
 
%coverage probability
temp = (IShat-(IS*ones(1,N_sim)) > 0);
cov_prob = (sum(temp))/N_sim;
lenght = mean(IShat);

tempmomI = (CSmomI-(IS*ones(1,N_sim)) > 0);
cov_probmomI = (sum(tempmomI))/N_sim;
lenghtmomI = mean(CSmomI);

tempmomO = (CSmomO-(IS*ones(1,N_sim)) > 0);
cov_probmomO = (sum(tempmomO))/N_sim;
lenghtmomO = mean(CSmomO);

tempmomBI = (CSB_I(2,:)-(IS*ones(1,N_sim)) > 0);
cov_probBI = (sum(tempmomBI))/N_sim;
lenghtBI = mean(CSB_I(2,:)-CSB_I(1,:));

tempmomBO = (CSB_O(2,:)-(IS*ones(1,N_sim)) > 0);
cov_probBO = (sum(tempmomBO))/N_sim;
lenghtBO = mean(CSB_O(2,:)-CSB_O(1,:));

tempqO_u = (CSq_O-(Fq_u*ones(1,N_sim)) > 0);
cov_probqO = (sum(tempqO_u))/N_sim;
lenghtqO = mean(arc_O(2,:)-arc_O(1,:));

tempqI_u = (CSq_I-(Fq_u*ones(1,N_sim)) > 0);
cov_probqI = (sum(tempqI_u))/N_sim;
lenghtqI = mean(arc_I(2,:)-arc_I(1,:));


Table = [{'Table ','cov prob', 'lenght'};
    'Mom Omega',num2cell(cov_probmomO),num2cell(lenghtmomO);
    'Mom Iden',num2cell(cov_probmomI), num2cell(lenghtmomI);
    'Bonf Omega', num2cell(cov_probBO), num2cell(lenghtBO);
    'Bonf Iden', num2cell(cov_probBI), num2cell(lenghtBI);
    'CSphi',num2cell(CS_phi),'_';
    'CSq Omega',num2cell(cov_probqO),num2cell(lenghtqO);
    'CSq I',num2cell(cov_probqI),num2cell(lenghtqI)]